
#~ from _wrap import _func, _register_funcs

import ctypes
class _func:
    '''Function definition, replaced by reference to function in library
    when register_functions() is called.'''
    def __init__(self, *args):
        self.args = args


def _register_funcs(libname, namespace):
    lib = ctypes.CDLL(libname)

    for name, fdef in namespace.items():
        if isinstance(fdef, _func):
            func = getattr(lib, name)
            func.restype = fdef.args[0]
            func.argtypes = fdef.args[1:]
            namespace[name] = func


from ctypes import POINTER, c_int, c_uint8, c_uint16, c_uint32, c_char_p, Structure


SLOG2_MAX_BUFFERS = 4

# Severity level definitions
SLOG2_SHUTDOWN  = 0   # Shut down the system NOW. eg: for OEM use
SLOG2_CRITICAL  = 1   # Unexpected unrecoverable error. eg: hard disk error
SLOG2_ERROR     = 2   # Unexpected recoverable error. eg: needed to reset a hw controller
SLOG2_WARNING   = 3   # Expected error. eg: parity error on a serial port
SLOG2_NOTICE    = 4   # Warnings. eg: Out of paper
SLOG2_INFO      = 5   # Information. eg: Printing page 3
SLOG2_DEBUG1    = 6   # Debug messages eg: Normal detail
SLOG2_DEBUG2    = 7   # Debug messages eg: Fine detail

SLOG2_NOTIFICATION_PATH = "/pps/services/slogger2/notify"

# Enum for the state of the buffer_set
NOTIFY_STATE_CONNECTED  = 0
NOTIFY_STATE_DISCONNECT = 1
NOTIFY_STATE_REMOVE     = 2

class slog2_buffer_meta(Structure):
    pass

# Slog2 buffer is an opaque handle
slog2_buffer_t = POINTER(slog2_buffer_meta)


# Struct containing the configuration data for a slog2 buffer.
class slog2_buffer_config_t(Structure):
    _fields_ = [
        ('buffer_name', c_char_p),  # What we want to name the buffer
        ('num_pages', c_int),       # The number of 4K pages this buffer contains
        ]

# Struct containing the configuration data for a slog2 buffer set. Each
# buffer set contains one or more buffers.  The size of each buffer is
# specified as a multiple of 4k pages.  The maximum number of buffers
# is specified by SLOG2_MAX_BUFFERS.
class slog2_buffer_set_config_t(Structure):
    _fields_ = [
        ('num_buffers', c_int),         # Number of buffers to configure
        ('buffer_set_name', c_char_p),  # Process name, or other descriptor
        ('verbosity_level', c_uint8),   # The minimum severity to log
        ('buffer_config', slog2_buffer_config_t * SLOG2_MAX_BUFFERS), # Buffer configuration for num_buffers
        ]


#------------------------------------------------
# Log a constant string to the slog2 buffer.
# @param buffer Handle of the selected buffer to log into, can be NULL only
#               if slog2_set_default_buffer(...) was previously called
#  @param code User-specified code, not used internally by slog2
#  @param severity The severity level of this log item
#  @param data Pointer to a constant string to log to the buffer
#  @return -1 on error, 0 on success
#
slog2c = _func(c_int, slog2_buffer_t, c_uint16, c_uint8, c_char_p)
#~ int slog2c( slog2_buffer_t __buffer,
            #~ _Uint16t __code,
            #~ _Uint8t __severity,
            #~ const char* __data );


#------------------------------------------------
#  Adjusts the verbosity of the# entire buffer_set* that this buffer belongs
#  to. This value overrides the value set in the PPS object and updates
#  the PPS value.
#
#  @param buffer Handle to the buffer that belongs to the buffer_set that
#                will have its verbosity adjusted
#  @param verbosity The new verbosity setting
#  @return -1 on error, 0 on success
#
slog2_set_verbosity = _func(c_int, slog2_buffer_t, c_uint8)
#~ int slog2_set_verbosity( slog2_buffer_t __buffer,
                         #~ _Uint8t __verbosity );


#------------------------------------------------
#  Returns the verbosity of the# entire buffer_set* that this buffer belongs
#  to.
#
#  @param buffer Handle to the buffer that belongs to the buffer_set that
#                will have its verbosity adjusted
#  @return the verbosity of the buffer_set
#
slog2_get_verbosity = _func(c_uint8, slog2_buffer_t)
#~ _Uint8t slog2_get_verbosity( slog2_buffer_t __buffer );


#------------------------------------------------
#  Register a new instance of a slog2 buffer set.  Each buffer set contains one or more buffers.
#  The size of each buffer is specified as a multiple of 4k pages.  The maximum number of
#  buffers is specified by SLOG2_MAX_BUFFERS.
#
#  @param config Pointer to a slog2 buffer configuration object
#  @param handles An array of handles to the buffers allocated
#  @param flags Bit OR of options for buffer management
#  @return -1 on error and handle data not guaranteed, 0 on success
#
slog2_register = _func(c_int, POINTER(slog2_buffer_set_config_t), POINTER(slog2_buffer_t), c_uint32)
#~ int slog2_register( const slog2_buffer_set_config_t# __config,
                    #~ slog2_buffer_t# __handles,
                    #~ _Uint32t __flags );


#------------------------------------------------
#  Sets the default logging buffer. When the slog2 logging APIs are
#  called with a NULL buffer handle the log will be written to the default
#  logging buffer set by this function.
#
#  @param buffer Handle of the buffer where default logs will go
#                If buffer is NULL, default logging will be
#                disabled. If buffer is -1 the default buffer handle is returned.
#  @return previous default buffer handle
#
slog2_set_default_buffer = _func(slog2_buffer_t, slog2_buffer_t)
#~ slog2_buffer_t slog2_set_default_buffer( slog2_buffer_t __buffer );


#------------------------------------------------
#  Resets the state of slogger2 within the process. All resources used by
#  slogger2 within the process are released so any existing buffer
#  handle(s) will be invalid.
#
#  @return -1 on error, 0 on success
#
slog2_reset = _func(c_int)
#~ int slog2_reset( void );


_register_funcs('libslog2.so', globals())
