'''BlackBerry-Tart support code including Application class.'''

import os
import sys
import json
import pickle

import _tart


def send(type, **kwargs):
    _tart.send(json.dumps([type, kwargs]))


def wait():
    data = _tart.wait()
    try:
        msg = json.loads(data)
    except ValueError:
        msg = {}
    return msg


def log(*args):
    '''Relies on the slogger2 configuration to send this to the right place.'''
    print(*args)



class Application:
    def __init__(self, debug=False):
        self.debug = debug
        if self.debug:
            log('tart: app starting')

        super(Application, self).__init__()


    def start(self):
        '''entry point to main loop'''
        send('backendReady')

        self.loop()


    def loop(self):
        '''main loop'''
        empty_dict = {}

        while True:
            msg = wait()
            if self.debug:
                log('tart: msg', msg)

            try:
                msg_type = msg[0]
                # apply Qt-style case normalization
                name = 'on' + msg_type[0].upper() + msg_type[1:]
            except IndexError:
                log('tart: ERROR, no type found in message')
                continue

            else:
                try:
                    handler = getattr(self, name)
                except AttributeError:
                    if msg_type.startswith('on'):
                        log('tart: WARNING, message starts with "on", maybe remove?')

                    self.missing_handler(msg)
                    continue

            try:
                # log('calling', handler)
                try:
                    kwargs = msg[1] or empty_dict
                except KeyError:
                    kwargs = empty_dict
                result = handler(**kwargs)
            except SystemExit:
                raise
            except:
                import traceback
                traceback.print_exception(*sys.exc_info())


    def missing_handler(self, msg):
        log('tart: ERROR, missing handler for', msg[0])


    def restore_data(self, data, path):
        '''retrieve persisted data, if any, and restore only those
        items we currently support'''
        try:
            saved = pickle.load(open(path, 'rb'))
        except:
            saved = {}

        # restore only recognized items, which means we'll ignore the
        # version key for now
        for key in data:
            try:
                data[key] = saved[key]
                log('{}: restored {} = {!r}'.format(
                    path,
                    key,
                    data[key],
                    ))
            except KeyError:
                pass


    def save_data(self, data, path):
        # we can get fancier later when we need
        data['version'] = 1

        # log('{}: persisting {!r}'.format(path, data))
        pickle.dump(data, open(path, 'wb'))


    def onUiReady(self):
        '''Sent when the QML has finished onCreationCompleted for the root
        component.  Override in subclasses as required.'''
        pass


    def onManualExit(self):
        '''Sent when the app is exiting, so we can save state etc.
        Override in subclasses as required.'''
        send('continueExit')


    #---------------------------------------------
    # useful things for debugging with:

    def js(self, text):
        '''send to JavaScript for evaluation'''
        send('evalJavascript', text=text)


    def onEvalResult(self, value=None):
        '''result from JavaScript eval sent from js()'''
        print('result', value)


# EOF
